package de.mdelab.mlsdm.interpreter;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.expressions.interpreter.core.ExpressionInterpreterManager;
import de.mdelab.expressions.interpreter.core.variables.VariablesScope;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.ActivityNode;
import de.mdelab.mlsdm.interpreter.facade.MLSDMMetamodelFacadeFactory;
import de.mdelab.mlsdm.interpreter.notifications.MLSDMNotificationEmitter;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.sdm.interpreter.core.notifications.NotificationEmitter;
import de.mdelab.sdm.interpreter.core.notifications.Notifier;

public class MLSDMExpressionInterpreterManager extends ExpressionInterpreterManager<EClassifier, EStructuralFeature, MLExpression> implements Notifier<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> {

	private final NotificationEmitter<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> notificationEmitter;
	
	public MLSDMExpressionInterpreterManager( 	final ClassLoader classLoader,
												final NotificationEmitter<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> notificationEmitter ) {
		super( MLSDMMetamodelFacadeFactory.INSTANCE.getExpressionFacade(), classLoader );
		
		this.notificationEmitter = notificationEmitter;
	}

	public MLSDMExpressionInterpreterManager(final ClassLoader classLoader)	{
		this(classLoader, new MLSDMNotificationEmitter());
	}

	@Override
	public NotificationEmitter<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> getNotificationEmitter() {
		return notificationEmitter;
	}

	@Override
	protected void evaluatingExpression(MLExpression expression,
			VariablesScope<EClassifier, ?, MLExpression> variablesScope) {
		getNotificationEmitter().evaluatingExpression( expression, variablesScope, this );
	}

	@Override
	protected void evaluatedExpression(MLExpression expression, Object value,
			VariablesScope<EClassifier, ?, MLExpression> variablesScope) {
		getNotificationEmitter().evaluatedExpression( expression, value, variablesScope, this );
	}
}
