package de.mdelab.mlsdm.interpreter.eclipse;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.expressions.interpreter.core.ExpressionsInterpreterException;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.ActivityNode;
import de.mdelab.mlsdm.interpreter.MLSDMInterpreter;
import de.mdelab.mlsdm.interpreter.facade.MLSDMMetamodelFacadeFactory;
import de.mdelab.mlsdm.interpreter.notifications.MLSDMNotificationEmitter;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.sdm.interpreter.core.eclipse.MLSDMEclipseExpressionsInterpreterManager;
import de.mdelab.sdm.interpreter.core.facade.MetamodelFacadeFactory;

/**
 * Implementation of the SDM interpreter for the MLSDM metamodel and use in
 * Eclipse.
 * 
 * @author Stephan Hildebrandt
 * 
 */
public class MLSDMEclipseInterpreter extends MLSDMInterpreter {

	public MLSDMEclipseInterpreter(	final ClassLoader classLoader,
									final MLSDMNotificationEmitter notificationEmitter,
									final MetamodelFacadeFactory<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> facadeFactory )
	throws ExpressionsInterpreterException {
		super(	new MLSDMEclipseExpressionsInterpreterManager<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>( MLSDMMetamodelFacadeFactory.INSTANCE.getExpressionFacade(), classLoader, notificationEmitter ).registerExpressionInterpreters(),
				notificationEmitter,
				facadeFactory );
	}

	public MLSDMEclipseInterpreter( final ClassLoader classLoader )
	throws ExpressionsInterpreterException {
		this( classLoader, new MLSDMNotificationEmitter(), MLSDMMetamodelFacadeFactory.INSTANCE );
	}

	public MLSDMEclipseInterpreter(	final ClassLoader classLoader,
									final MetamodelFacadeFactory<Activity, ActivityNode, ActivityEdge, StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> p_facadeFactory )
	throws ExpressionsInterpreterException {
		this( classLoader, new MLSDMNotificationEmitter(), p_facadeFactory );
	}
}
