package de.mdelab.mlsdm.interpreter.eclipse.searchModel;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.expressions.interpreter.core.IExpressionsFacade;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.expressions.ExpressionAnalyzer;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.expressions.ExpressionAnalyzerManager;

public class MLSDMEclipseExpressionAnalyzerManager extends
		ExpressionAnalyzerManager<EClassifier, EStructuralFeature, MLExpression> {

	private static final String EXPRESSION_ANALYZERS_EXTENSION_POINT_ID 	= "de.mdelab.mlsdm.interpreter.eclipse.analyzer";
	private static final String EXPRESSION_LANGUAGE_ATTRIBUTE_NAME 			= "language";
	private static final String EXPRESSION_INTERPRETER_CLASS_ATTRIBUTE_NAME = "class";

	public MLSDMEclipseExpressionAnalyzerManager(IExpressionsFacade<MLExpression> expressionFacade) throws SDMException {
		super(expressionFacade);
		
		/*
		 * register available expression analyzers
		 */
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IConfigurationElement[] configurationElements = registry.getConfigurationElementsFor(EXPRESSION_ANALYZERS_EXTENSION_POINT_ID);

		for (IConfigurationElement configElement : configurationElements)
		{
			String expressionLanguage = configElement.getAttribute(EXPRESSION_LANGUAGE_ATTRIBUTE_NAME);

			try
			{
				@SuppressWarnings("unchecked")
				ExpressionAnalyzer<EClassifier, EStructuralFeature, MLExpression> expressionAnalyzer =
					(ExpressionAnalyzer<EClassifier, EStructuralFeature, MLExpression>) configElement
						.createExecutableExtension(EXPRESSION_INTERPRETER_CLASS_ATTRIBUTE_NAME);

				this.registerExpressionAnalyzer(expressionLanguage, expressionAnalyzer);

			}
			catch (CoreException e)
			{
				e.printStackTrace();

				throw new SDMException("Could not instantiate expression analyzer for language '" + expressionLanguage + "'.", e);
			}

		}
	}

}
