package de.mdelab.mlsdm.interpreter.patternMatcher.patternPartBased;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;

import de.mdelab.expressions.interpreter.core.variables.Variable;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.MapEntryLink;
import de.mdelab.mlstorypatterns.MatchTypeEnum;
import de.mdelab.mlstorypatterns.ModifierEnum;
import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.ECheckResult;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.EMatchType;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.MatchState;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.PatternPart;
import de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased.PatternPartBasedMatcher;

/**
 * This pattern part matches and manipulates MapEntryLinks, i.e. key-value
 * mappings.
 * 
 * @author Stephan Hildebrandt
 * 
 */
public class MLSDMMapEntryLinkPatternPart extends MLSDMPatternPart<AbstractStoryPatternObject, MapEntryLink>
{

	public MLSDMMapEntryLinkPatternPart(
			final PatternPartBasedMatcher<?, ?, ?, ?, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, ?, MLExpression> patternMatcher,
			final MapEntryLink storyPatternLink)
	{
		super(patternMatcher, storyPatternLink, storyPatternLink.getValueTarget() == null ? new AbstractStoryPatternObject[]
		{
				storyPatternLink.getSource(), storyPatternLink.getTarget()
		} : new AbstractStoryPatternObject[]
		{
				storyPatternLink.getSource(), storyPatternLink.getTarget(), storyPatternLink.getValueTarget()
		});

		/*
		 * if a story pattern object is created, the link must also be created
		 */
		assert storyPatternLink.getSource().getModifier() != ModifierEnum.CREATE || storyPatternLink.getModifier() == ModifierEnum.CREATE;
		assert storyPatternLink.getTarget().getModifier() != ModifierEnum.CREATE || storyPatternLink.getModifier() == ModifierEnum.CREATE;

		/*
		 * if a story pattern object is destroyed, the link must also be
		 * destroyed
		 */
		assert storyPatternLink.getSource().getModifier() != ModifierEnum.DESTROY || storyPatternLink.getModifier() == ModifierEnum.DESTROY;
		assert storyPatternLink.getTarget().getModifier() != ModifierEnum.DESTROY || storyPatternLink.getModifier() == ModifierEnum.DESTROY;

		/*
		 * If one of the link ends is optional, the links must be optional as
		 * well.
		 */
		assert storyPatternLink.getSource().getMatchType() != MatchTypeEnum.OPTIONAL
				|| storyPatternLink.getMatchType() == MatchTypeEnum.OPTIONAL;
		assert storyPatternLink.getTarget().getMatchType() != MatchTypeEnum.OPTIONAL
				|| storyPatternLink.getMatchType() == MatchTypeEnum.OPTIONAL;

		/*
		 * If the link is created or destroyed, the link's feature must be
		 * changeable
		 */
		assert storyPatternLink.getModifier() != ModifierEnum.CREATE && storyPatternLink.getModifier() != ModifierEnum.DESTROY
				|| storyPatternLink.getFeature().isChangeable();

		if (storyPatternLink.getValueTarget() != null)
		{
			assert storyPatternLink.getValueTarget().getModifier() != ModifierEnum.CREATE
					|| storyPatternLink.getModifier() == ModifierEnum.CREATE;

			assert storyPatternLink.getValueTarget().getModifier() != ModifierEnum.DESTROY
					|| storyPatternLink.getModifier() == ModifierEnum.DESTROY;

			assert storyPatternLink.getValueTarget().getMatchType() != MatchTypeEnum.OPTIONAL
					|| storyPatternLink.getMatchType() == MatchTypeEnum.OPTIONAL;
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void doCreateLink()
	{
		if (this.storyPatternLink.getModifier() == ModifierEnum.CREATE)
		{
			final Variable<EClassifier> sourceVariable = this.patternMatcher.getVariablesScope().getVariable(
					this.storyPatternLink.getSource().getName());
			final Variable<EClassifier> targetVariable = this.patternMatcher.getVariablesScope().getVariable(
					this.storyPatternLink.getTarget().getName());

			assert sourceVariable != null;
			assert targetVariable != null;

			assert sourceVariable.getValue() instanceof EObject;

			final EObject sourceEObject = (EObject) sourceVariable.getValue();

			if (this.storyPatternLink.getValueTarget() == null)
			{
				((EMap<Object, Object>) sourceEObject.eGet(this.storyPatternLink.getFeature())).put(targetVariable.getValue(), null);

				this.patternMatcher.getNotificationEmitter().instanceLinkCreated(this.storyPatternLink.getSource(), sourceEObject,
						this.storyPatternLink, this.storyPatternLink.getTarget(), targetVariable.getValue(),
						this.patternMatcher.getVariablesScope(), this.patternMatcher);
			}
			else
			{
				final Variable<EClassifier> valueTargetVariable = this.patternMatcher.getVariablesScope().getVariable(
						this.storyPatternLink.getValueTarget().getName());

				assert valueTargetVariable != null;

				((EMap<Object, Object>) sourceEObject.eGet(this.storyPatternLink.getFeature())).put(targetVariable.getValue(),
						valueTargetVariable.getValue());

				this.patternMatcher.getNotificationEmitter().instanceLinkCreated(this.storyPatternLink.getSource(), sourceEObject,
						this.storyPatternLink, this.storyPatternLink.getTarget(), targetVariable.getValue(),
						this.patternMatcher.getVariablesScope(), this.patternMatcher);

				this.patternMatcher.getNotificationEmitter().instanceLinkCreated(this.storyPatternLink.getSource(), sourceEObject,
						this.storyPatternLink, this.storyPatternLink.getValueTarget(), valueTargetVariable.getValue(),
						this.patternMatcher.getVariablesScope(), this.patternMatcher);
			}
		}
	}

	@Override
	protected void doDestroyLink(final Map<AbstractStoryPatternObject, Object> deletedObjects)
	{
		assert this.storyPatternLink.getModifier() == ModifierEnum.DESTROY;

		final AbstractStoryPatternObject sourceSpo = this.storyPatternLink.getSource();
		final AbstractStoryPatternObject targetSpo = this.storyPatternLink.getTarget();

		assert deletedObjects.get(sourceSpo) == null || deletedObjects.get(sourceSpo) instanceof EObject;

		EObject sourceInstanceObject = (EObject) deletedObjects.get(sourceSpo);
		Object targetInstanceObject = deletedObjects.get(targetSpo);

		/*
		 * The source was not deleted or it is optional and was not matched.
		 */
		if (sourceInstanceObject == null)
		{
			final Variable<EClassifier> sourceVariable = this.patternMatcher.getVariablesScope().getVariable(sourceSpo.getName());

			if (sourceVariable != null)
			{
				assert sourceSpo.getMatchType() != MatchTypeEnum.OPTIONAL;
				assert sourceVariable.getValue() instanceof EObject;

				sourceInstanceObject = (EObject) sourceVariable.getValue();
			}
		}

		/*
		 * The target was not deleted
		 */
		if (targetInstanceObject == null)
		{
			final Variable<EClassifier> targetVariable = this.patternMatcher.getVariablesScope().getVariable(targetSpo.getName());

			if (targetVariable != null)
			{
				assert targetSpo.getMatchType() != MatchTypeEnum.OPTIONAL;

				targetInstanceObject = targetVariable.getValue();
			}
		}

		/*
		 * Delete the link
		 */
		if (sourceInstanceObject != null && targetInstanceObject != null)
		{
			@SuppressWarnings("unchecked")
			final Object value = ((EMap<Object, Object>) sourceInstanceObject.eGet(this.storyPatternLink.getFeature()))
					.removeKey(targetInstanceObject);

			this.patternMatcher.getNotificationEmitter().instanceLinkDestroyed(sourceSpo, sourceInstanceObject, this.storyPatternLink,
					targetSpo, targetInstanceObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

			if (this.storyPatternLink.getValueTarget() != null)
			{
				this.patternMatcher.getNotificationEmitter().instanceLinkDestroyed(sourceSpo, sourceInstanceObject, this.storyPatternLink,
						this.storyPatternLink.getValueTarget(), value, this.patternMatcher.getVariablesScope(), this.patternMatcher);
			}
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public ECheckResult check() throws SDMException
	{
		/*
		 * To check a map entry link, the source and key objects must be bound.
		 * The value object may still be unbound.
		 */
		if (this.storyPatternLink.getModifier() != ModifierEnum.CREATE && this.patternMatcher.isBound(this.storyPatternLink.getSource())
				&& this.patternMatcher.isBound(this.storyPatternLink.getTarget()))
		{
			final AbstractStoryPatternObject sourceSpo = this.storyPatternLink.getSource();
			final AbstractStoryPatternObject keySpo = this.storyPatternLink.getTarget();
			final AbstractStoryPatternObject valueSpo = this.storyPatternLink.getValueTarget();

			final EObject sourceInstanceObject = (EObject) this.patternMatcher.getInstanceObject(sourceSpo);
			final Object keyInstanceObject = this.patternMatcher.getInstanceObject(keySpo);

			/*
			 * There is no value or the value is unbound.
			 */
			if (valueSpo == null || !this.patternMatcher.isBound(valueSpo))
			{
				if (((EMap<Object, Object>) sourceInstanceObject.eGet(this.storyPatternLink.getFeature())).containsKey(keyInstanceObject))
				{
					this.patternMatcher.getNotificationEmitter().linkCheckSuccessful(sourceSpo, sourceInstanceObject,
							this.storyPatternLink, keySpo, keyInstanceObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					if (valueSpo != null)
					{
						/*
						 * Return unknown if there is an unbound value.
						 */
						return ECheckResult.UNKNOWN;
					}
					else
					{
						return ECheckResult.OK;
					}
				}
				else
				{
					this.patternMatcher.getNotificationEmitter().linkCheckFailed(sourceSpo, sourceInstanceObject, this.storyPatternLink,
							keySpo, keyInstanceObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					return ECheckResult.FAIL;
				}
			}
			else
			{
				final Object valueObject = this.patternMatcher.getInstanceObject(valueSpo);

				if (valueObject == ((EMap<Object, Object>) sourceInstanceObject.eGet(this.storyPatternLink.getFeature()))
						.get(keyInstanceObject))
				{
					this.patternMatcher.getNotificationEmitter().linkCheckSuccessful(sourceSpo, sourceInstanceObject,
							this.storyPatternLink, keySpo, keyInstanceObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					this.patternMatcher.getNotificationEmitter().linkCheckSuccessful(sourceSpo, sourceInstanceObject,
							this.storyPatternLink, valueSpo, valueObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					return ECheckResult.OK;
				}
				else
				{
					this.patternMatcher.getNotificationEmitter().linkCheckFailed(sourceSpo, sourceInstanceObject, this.storyPatternLink,
							keySpo, keyInstanceObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					this.patternMatcher.getNotificationEmitter().linkCheckFailed(sourceSpo, sourceInstanceObject, this.storyPatternLink,
							valueSpo, valueObject, this.patternMatcher.getVariablesScope(), this.patternMatcher);

					return ECheckResult.FAIL;
				}
			}
		}
		else if (this.storyPatternLink.getModifier() == ModifierEnum.CREATE
				&& this.storyPatternLink.getMatchType() != MatchTypeEnum.OPTIONAL)
		{
			return ECheckResult.OK;
		}
		else
		{
			return ECheckResult.UNKNOWN;
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean match(final MatchState matchState) throws SDMException
	{
		assert matchState instanceof MLSDMStoryPatternLinkMatchState;

		final MLSDMStoryPatternLinkMatchState<Iterator<Entry<Object, Object>>> ms = (MLSDMStoryPatternLinkMatchState<Iterator<Entry<Object, Object>>>) matchState;

		final AbstractStoryPatternObject sourceSpo = this.storyPatternLink.getSource();
		final AbstractStoryPatternObject keySpo = this.storyPatternLink.getTarget();
		final AbstractStoryPatternObject valueSpo = this.storyPatternLink.getValueTarget();

		assert sourceSpo != null;
		assert keySpo != null;

		assert this.patternMatcher.isBound(sourceSpo);

		final EObject sourceInstanceObject = (EObject) this.patternMatcher.getInstanceObject(sourceSpo);
		
		// DB ticket #1136
		assert  ms.getSourceInstanceObject() == null || sourceInstanceObject == ms.getSourceInstanceObject() : "Source instance object should be the same as that of match state!";

		/*
		 * There may be two cases: 1. The key object is unbound, the value
		 * object may not exist or be bound or unbound. 2. The key object is
		 * already bound, the value object must exist and be unbound.
		 */
		if (!this.patternMatcher.isBound(keySpo))
		{
			this.patternMatcher.getNotificationEmitter().traversingLink(this.storyPatternLink, sourceSpo, sourceInstanceObject, keySpo,
					this.patternMatcher.getVariablesScope(), this.patternMatcher);

			Iterator<Entry<Object, Object>> iterator = ms.getLinkIterator();

			if (iterator == null ) {
				iterator = ((EMap<Object, Object>) sourceInstanceObject.eGet(this.storyPatternLink.getFeature())).entrySet().iterator();

				ms.setLinkIterator(iterator);
				ms.setSourceInstanceObject(sourceInstanceObject);
			}

			while (iterator.hasNext())
			{
				final Entry<Object, Object> entry = iterator.next();

				final Object keyObject = entry.getKey();
				final Object valueObject = entry.getValue();

				/*
				 * If there is a bound value object, compare it to this entry's
				 * value.
				 */
				if (valueSpo != null && this.patternMatcher.isBound(valueSpo))
				{
					if (this.patternMatcher.getInstanceObject(valueSpo) != valueObject)
					{
						continue;
					}
				}

				if (this.patternMatcher.matchStoryPatternObject(keySpo, keyObject))
				{
					this.patternMatcher.getNotificationEmitter().storyPatternObjectBound(keySpo, keyObject,
							this.patternMatcher.getVariablesScope(), this.patternMatcher);

					if (valueSpo != null && !this.patternMatcher.isBound(valueSpo))
					{
						this.patternMatcher.getNotificationEmitter().traversingLink(this.storyPatternLink, sourceSpo, sourceInstanceObject,
								valueSpo, this.patternMatcher.getVariablesScope(), this.patternMatcher);

						if (this.patternMatcher.matchStoryPatternObject(valueSpo, valueObject))
						{
							this.patternMatcher.getNotificationEmitter().storyPatternObjectBound(valueSpo, valueObject,
									this.patternMatcher.getVariablesScope(), this.patternMatcher);

							return true;
						}
						else
						{
							this.patternMatcher.getNotificationEmitter().storyPatternObjectNotBound(valueSpo,
									this.patternMatcher.getVariablesScope(), this.patternMatcher);

							/*
							 * roll back the match transaction of keySpo
							 */
							this.patternMatcher.rollBackLastMatchStoryPatternObjectTransaction();
						}
					}
					else
					{
						return true;
					}
				}
			}

			this.patternMatcher.getNotificationEmitter().storyPatternObjectNotBound(keySpo, this.patternMatcher.getVariablesScope(),
					this.patternMatcher);

			if (valueSpo != null && !this.patternMatcher.isBound(valueSpo))
			{
				this.patternMatcher.getNotificationEmitter().storyPatternObjectNotBound(valueSpo, this.patternMatcher.getVariablesScope(),
						this.patternMatcher);
			}
		}
		else
		{
			/*
			 * The key is bound, the value is unbound.
			 */
			assert valueSpo != null;
			assert !this.patternMatcher.isBound(valueSpo);

			final Object keyInstanceObject = this.patternMatcher.getInstanceObject(keySpo);

			this.patternMatcher.getNotificationEmitter().traversingLink(this.storyPatternLink, sourceSpo, sourceInstanceObject, valueSpo,
					this.patternMatcher.getVariablesScope(), this.patternMatcher);

			if (this.patternMatcher.matchStoryPatternObject(valueSpo,
					((EMap<Object, Object>) sourceInstanceObject.eGet(this.storyPatternLink.getFeature())).get(keyInstanceObject)))
			{
				return true;
			}

			this.patternMatcher.getNotificationEmitter().storyPatternObjectNotBound(valueSpo, this.patternMatcher.getVariablesScope(),
					this.patternMatcher);
		}

		return false;
	}

	@SuppressWarnings("unchecked")
	@Override
	public int calculateMatchingCost()
	{
		assert !(this.patternMatcher.isBound(this.storyPatternLink.getSource())
				&& this.patternMatcher.isBound(this.storyPatternLink.getTarget()) && this.patternMatcher.isBound(this.storyPatternLink
				.getValueTarget()));

		if (this.patternMatcher.isBound(this.storyPatternLink.getSource()))
		{
			/*
			 * If the key is already bound, the matching cost is very low.
			 */
			if (this.patternMatcher.isBound(this.storyPatternLink.getTarget()))
			{
				return 2 + this.getNumberOfIncomingUnidirectionalLinks(this.storyPatternLink.getValueTarget());
			}
			else
			{
				/*
				 * Otherwise, the cost is the number of map entries.
				 */
				final Variable<EClassifier> sourceVariable = this.patternMatcher.getVariablesScope().getVariable(
						this.storyPatternLink.getSource().getName());

				assert sourceVariable.getValue() instanceof EObject;

				final EObject sourceEObject = (EObject) sourceVariable.getValue();

				return ((EMap<Object, Object>) sourceEObject.eGet(this.storyPatternLink.getFeature())).size()
						+ this.getNumberOfIncomingUnidirectionalLinks(this.storyPatternLink.getTarget());
			}
		}
		else
		{
			return PatternPart.MATCHING_NOT_POSSIBLE;
		}
	}

	@Override
	public EMatchType doGetMatchType()
	{
		switch (this.storyPatternLink.getModifier())
		{
			case NONE:
				switch (this.storyPatternLink.getMatchType())
				{
					case DEFAULT:
						return EMatchType.MANDATORY;
					case OPTIONAL:
						return EMatchType.OPTIONAL;
				}
			case CREATE:
				switch (this.storyPatternLink.getMatchType())
				{
					case DEFAULT:
						return EMatchType.OPTIONAL;
					case OPTIONAL:
						return EMatchType.OPTIONAL;
				}
			case DESTROY:
				switch (this.storyPatternLink.getMatchType())
				{
					case DEFAULT:
						return EMatchType.MANDATORY;
					case OPTIONAL:
						return EMatchType.OPTIONAL;
				}
			default:
				throw new UnsupportedOperationException();
		}
	}

	@Override
	public MatchState createMatchState()
	{
		return new MLSDMStoryPatternLinkMatchState<Iterator<Entry<Object, Object>>>();
	}
}
