package de.mdelab.mlsdm.interpreter.searchModel.model.action;

import java.util.Map;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.interpreter.searchModel.model.MLSDMCheckMatchingAction;
import de.mdelab.mlsdm.interpreter.searchModel.patternMatcher.MLSDMSearchModelBasedMatcher;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.CheckOnlyMatchingTransaction;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.MatchState;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.MatchingAction;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternConstraint;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternNode;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.SearchModelMatchingTransaction;

/**
 * This is a matching action for executing an explicit
 * check for a domain (type) constraint. It can be executed
 * once a mapping has been created for the node with the
 * type constraint.
 * 
 * @author Matthias.Barkowsky
 *
 */
public class MLSDMDomainCheckMatchingAction extends
		MLSDMCheckMatchingAction {

	/*
	 * Node over which the domain constraint is formulated.
	 */
	private PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> patternNode;

	public MLSDMDomainCheckMatchingAction(int id,
			MLSDMSearchModelBasedMatcher matcher,
			PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> patternNode) {
		super(id, matcher);
		this.patternNode = patternNode;
		this.addRequiredBinding(patternNode);
	}

	/* (non-Javadoc)
	 * @see de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.CheckMatchingAction#check()
	 */
	@Override
	public boolean check() {
		return true;
	}

	@Override
	public MatchState createMatchState() {
		if(!patternNode.isBound()) {
			return null;
		}
		return new MLSDMCheckOnceMatchState();
	}

	/* (non-Javadoc)
	 * @see de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.MatchingAction#doEstimateCardinality()
	 */
	@Override
	protected double doEstimateCardinality() {
		if(!patternNode.isBound()) {
			return PatternConstraint.MATCHING_NOT_POSSIBLE;
		}
		return PatternConstraint.CHECK_ONLY;
	}

	/* (non-Javadoc)
	 * @see de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.MatchingAction#match(de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.MatchState)
	 */
	@Override
	public SearchModelMatchingTransaction<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> match(
			MatchState matchState) {
		CheckOnlyMatchingTransaction<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> transaction =
				new CheckOnlyMatchingTransaction<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>(getPatternConstraint());
		
		if(!(matchState instanceof MLSDMCheckOnceMatchState)) {
			transaction.setValid(false);
		}
		else {
			MLSDMCheckOnceMatchState ms = (MLSDMCheckOnceMatchState) matchState;
			
			transaction.setValid(ms.check());
		}
		return transaction;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "check\t\t(" + patternNode.getSpo().getName() + ": " + patternNode.getSpo().getType().getName() + ")";
	}

	@SuppressWarnings("unchecked")
	@Override
	public MatchingAction<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> clonePristine(
			PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> clonePatternConstraint,
			Map<Object, Object> clonePNs) {
		return new MLSDMDomainCheckMatchingAction(id, matcher, (PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>) clonePNs.get(patternNode));
	}

}
