package de.mdelab.mlsdm.interpreter.searchModel.model.action;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.interpreter.searchModel.model.MLSDMPatternConstraint;
import de.mdelab.mlsdm.interpreter.searchModel.patternMatcher.MLSDMSearchModelBasedMatcher;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.LinkPositionConstraintEnum;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.mlstorypatterns.StoryPatternLink;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternNode;

/**
 * This is a matching action for executing an optimized
 * explicit check for a constraint based on a link. It makes
 * use of the data structures provided by a reference adapter.
 * 
 * @author Matthias.Barkowsky
 *
 */
public class MLSDMLinkCheckOptimizedMatchingAction extends MLSDMLinkCheckMatchingAction {

	public MLSDMLinkCheckOptimizedMatchingAction(int id,
			MLSDMSearchModelBasedMatcher matcher, MLSDMPatternConstraint patternConstraint, StoryPatternLink link,
			PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> sourceNode,
			PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> targetNode) {
		super(id, matcher, patternConstraint, link, sourceNode, targetNode);
	}
	
	/* (non-Javadoc)
	 * @see de.mdelab.mlsdm.interpreter.patternMatcher.searchModelBased.searchModel.action.MLSDMLinkCheckMatchingAction#checkLink()
	 */
	@Override
	protected boolean checkLink() {
		EReference feature = (EReference) link.getFeature();
		if(feature.isContainment() || !feature.isMany() || (link.getLinkPositionConstraint() != null && link.getLinkPositionConstraint() != LinkPositionConstraintEnum.UNCONSTRAINED)) {
			return super.checkLink();
		}
		
		boolean success;
		AbstractStoryPatternObject sourceSPO = link.getSource();
		AbstractStoryPatternObject targetSPO = link.getTarget();
		
		EObject source = (EObject) matcher.getVariable(sourceSPO.getName()).getValue();
		EObject target = (EObject) matcher.getVariable(targetSPO.getName()).getValue();
		
		success = matcher.getReferenceAdapter().checkReference(source, (EReference) feature, target);
		
		if(success) {
			matcher.getNotificationEmitter().linkCheckSuccessful(link.getSource(), source, link, link.getTarget(), target, matcher.getVariablesScope(), matcher);
		}
		else {
			matcher.getNotificationEmitter().linkCheckFailed(link.getSource(), source, link, link.getTarget(), target, matcher.getVariablesScope(), matcher);
		}
		
		return success;
	}

	/* (non-Javadoc)
	 * @see de.mdelab.mlsdm.interpreter.patternMatcher.searchModelBased.searchModel.action.MLSDMLinkCheckMatchingAction#toString()
	 */
	@Override
	public String toString() {
		return "check\t\t(" + link.getSource().getName() + " -" + link.getFeature().getName() + "-> " + link.getTarget().getName() + ")";
	}

}
