package de.mdelab.mlsdm.interpreter.searchModel.patternMatcher.expressions;
import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.ocl.ecore.utilities.AbstractVisitor;
import org.eclipse.ocl.expressions.PropertyCallExp;
import org.eclipse.ocl.expressions.VariableExp;

import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.expressions.FeatureAccess;

/**
 * This is an implementation of the visitor in the visitor pattern
 * which can visit the abstract syntax tree of an OCL expression
 * and collects the all accessed features of variables in the expression
 * in the course of the visiting process.
 * 
 * @author Matthias.Barkowsky
 *
 */
public class MLSDMOCLFeatureAccessCollectorVisitor extends AbstractVisitor<Object> {

	/*
	 * Collection of all collected variable names.
	 */
	private Collection<FeatureAccess<EStructuralFeature>> featureAccesses;
	
	public MLSDMOCLFeatureAccessCollectorVisitor() {
		super();
		featureAccesses = new ArrayList<FeatureAccess<EStructuralFeature>>();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ocl.utilities.AbstractVisitor#visitVariableExp(org.eclipse.ocl.expressions.VariableExp)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Object visitPropertyCallExp(PropertyCallExp<EClassifier, EStructuralFeature> callExp) {
		if(callExp.getSource() instanceof VariableExp) {
			featureAccesses.add(new FeatureAccess<EStructuralFeature>(((VariableExp<EClassifier, EParameter>) callExp.getSource()).getReferredVariable().getName(), callExp.getReferredProperty()));
		}
		
		return super.visitPropertyCallExp(callExp);
	}
	
	public Collection<FeatureAccess<EStructuralFeature>> getFeatureAccesses(){
		return featureAccesses;
	}

}
