package de.mdelab.mlsdm.interpreter.searchModel.patternMatcher.strategy;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.interpreter.searchModel.model.action.MLSDMDomainCheckMatchingAction;
import de.mdelab.mlsdm.interpreter.searchModel.model.action.MLSDMSPOExpressionCheckMatchingAction;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternConstraint;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.strategy.OrderProducingSelectionStrategy;

/**
 * This is a pattern constraint selection strategy emulating the
 * default behavior of the pattern part based matcher. It only
 * creates new mappings via edge traversal in forward direction,
 * while checking constraints formulated over single story pattern
 * objects as soon as a mapping for said object has been created,
 * and constraints formulated over the entire story pattern at the
 * very end of the matching.
 * 
 * @author Matthias.Barkowsky
 *
 */
public class MLSDMEmulateDefaultSelectionStrategy extends OrderProducingSelectionStrategy<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> {

	/*
	 * Strategy implementing the cost model used for edge
	 * constraints.
	 */
	protected MLSDMMatchingCostStrategy costStrategy;
	
	public MLSDMEmulateDefaultSelectionStrategy() {
		this.costStrategy = new MLSDMCardinalityMatchingCostStrategy();
	}	
	
	/* (non-Javadoc)
	 * @see de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.strategy.OrderProducingSelectionStrategy#selectNextConstraint()
	 */
	@Override
	protected PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> selectNextConstraint() {
		PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> cheapestConstraint = null;
		double minCost = PatternConstraint.MATCHING_NOT_POSSIBLE;
		
		PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> cheapestConstraintOptional = null;
		double minCostOptional = PatternConstraint.MATCHING_NOT_POSSIBLE;
		
		for(PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> patternConstraint:searchModel.getPatternConstraints()) {
			if(patternConstraint.isActive() || patternConstraint.getCardinality() == PatternConstraint.MATCHING_NOT_POSSIBLE) {
				continue;
			}
			
			double cost = !(patternConstraint.getConstraint() instanceof AbstractStoryPatternLink) && !(patternConstraint.getActiveAction() instanceof MLSDMSPOExpressionCheckMatchingAction) && !(patternConstraint.getActiveAction() instanceof MLSDMDomainCheckMatchingAction) ? PatternConstraint.ALWAYS_MATCH_LAST : costStrategy.calculateMatchingCost(patternConstraint);
			
			if(!patternConstraint.isOptional()) {
				if(cost < minCost) {
					cheapestConstraint = patternConstraint;
					minCost = cost;
				}
			}
			else {
				if(cost < minCostOptional) {
					cheapestConstraintOptional = patternConstraint;
					minCostOptional = cost;
				}
			}
		}
		
		return cheapestConstraint != null ? cheapestConstraint : cheapestConstraintOptional;
	}

}
