package de.mdelab.mlsdm.interpreter.searchModel.patternMatcher.strategy;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternConstraint;

/**
 * This is an implementation of the abstract strategy from the
 * strategy design pattern for encoding a matching cost model
 * for pattern constraints in the context of the search model
 * based execution of a graph query.
 * 
 * @author Matthias.Barkowsky
 *
 */
public interface MLSDMMatchingCostStrategy{
	
	/**
	 * Calculate the matching cost of the given pattern constraint
	 * according to this strategy's implemented cost model.
	 * 
	 * @param patternConstraint
	 * 			The pattern constraint.
	 * @return	The calculated matching cost.
	 */
	public double calculateMatchingCost(PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> patternConstraint);

	/**
	 * Perform initializations as necessary.
	 */
	public void initialize();

}
