package de.mdelab.mlsdm.interpreter.searchModel.patternMatcher.strategy;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.interpreter.searchModel.model.action.MLSDMExpressionCheckMatchingAction;
import de.mdelab.mlsdm.interpreter.searchModel.model.action.MLSDMIndexMatchingAction;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.MatchingAction;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternConstraint;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternNode;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.strategy.OrderProducingSelectionStrategy;

public class MLSDMNoUpdateSelectionStrategy
		extends
			OrderProducingSelectionStrategy<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> {

	private Set<PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>> checkedConstraints;
	private Set<PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>> uncheckedConstraints;

	private LinkedList<Collection<PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>> bindingStack;
	
	public MLSDMNoUpdateSelectionStrategy() {
		checkedConstraints = new HashSet<PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>();
		uncheckedConstraints = new HashSet<PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>();
		bindingStack = new LinkedList<Collection<PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>>();
	}
	
	@Override
	protected void doInitialize() {
		uncheckedConstraints.addAll(searchModel.getPatternConstraints());
	}
	
	@Override
	protected PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> doPopPatternConstraint() {
		PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> constraint = super.doPopPatternConstraint();
		checkedConstraints.add(constraint);
		uncheckedConstraints.remove(constraint);
		return constraint;
	}
	
	@Override
	protected PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> doRollBackLastPop() {
		PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> constraint = super.doRollBackLastPop();
		checkedConstraints.remove(constraint);
		uncheckedConstraints.add(constraint);
		return constraint;
	}
	
	@Override
	protected PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> selectNextConstraint() {
		PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> cheapest = null;
		double cheapestCost = PatternConstraint.MATCHING_NOT_POSSIBLE;
		for(PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> constraint:uncheckedConstraints) {
			MatchingAction<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> action = constraint.computeCheapestAction();
			if(action == null) {
				continue;
			}
			double matchingCost = (action instanceof MLSDMIndexMatchingAction || action instanceof MLSDMExpressionCheckMatchingAction) ? PatternConstraint.ALWAYS_MATCH_LAST : action.getCardinality();
			if(matchingCost == PatternConstraint.CHECK_ONLY) {
				constraint.setActiveAction(action);
				return constraint;
			}
			else if(matchingCost <= cheapestCost) {
				constraint.setActiveAction(action);
				cheapest = constraint;
				cheapestCost = action.getCardinality();
			}
		}
		return cheapest;
	}
	
	@Override
	protected void updateSearchModel(
			Collection<PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>> newlyBoundPatternNodes,
			boolean updateMatchingCosts) {
		for(PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> patternNode:newlyBoundPatternNodes) {
			patternNode.setBound(true);
		}
		bindingStack.push(newlyBoundPatternNodes);
	}
	
	@Override
	protected void rollBackSearchModel() {
		Collection<PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>> revertedBindings = bindingStack.pop();
		for(PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> patternNode:revertedBindings) {
			patternNode.setBound(false);
		}
	}

	@Override
	public void reset() {
		checkedConstraints = new HashSet<PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>();
		uncheckedConstraints = new HashSet<PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>();
		bindingStack = new LinkedList<Collection<PatternNode<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression>>>();
	}
}
