package de.mdelab.mlsdm.interpreter.searchModel.patternMatcher.strategy;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;

import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlstorypatterns.AbstractStoryPatternLink;
import de.mdelab.mlstorypatterns.AbstractStoryPatternObject;
import de.mdelab.mlstorypatterns.StoryPattern;
import de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternConstraint;

/**
 * This is a cost strategy encoding a cost model where the
 * matching cost is determined by the n-th root of the cardinality
 * of the pattern constraint's current matching action's result
 * set, where n is the number of pattern nodes for which the
 * matching operation produces candidate mappings.
 * 
 * @author Matthias.Barkowsky
 *
 */
public class MLSDMRootMatchingCostStrategy implements MLSDMMatchingCostStrategy {

	/* (non-Javadoc)
	 * @see de.mdelab.mlsdm.interpreter.patternMatcher.searchModelBased.strategy.cost.MLSDMMatchingCostStrategy#calculateMatchingCost(de.mdelab.sdm.interpreter.core.patternmatcher.searchModelBased.PatternConstraint)
	 */
	@Override
	public double calculateMatchingCost(
			PatternConstraint<StoryPattern, AbstractStoryPatternObject, AbstractStoryPatternLink, EClassifier, EStructuralFeature, MLExpression> patternConstraint) {
		int unmatchedNodes = patternConstraint.getDependencies().size() - patternConstraint.getMatchedDependencies();
		return (unmatchedNodes < 1 ? 1 : Math.pow(patternConstraint.getCardinality(), (1.0 / unmatchedNodes)));
	}

	@Override
	public void initialize() {
		
	}

}
